# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania

"""BesserWisser Product
"""
__version__='$Revision: 1.24 $'[11:-2]

# Python imports

# Zope imports
import Globals
import DateTime
from OFS.SimpleItem import SimpleItem
import AccessControl.Role
from Products.ZCatalog.CatalogAwareness import CatalogAware
from AccessControl import ClassSecurityInfo
from AccessControl.Permissions import view_management_screens, view

# Product imports
from Utils import Utils
import Resources

# Differences from BesserWisser 1.x questions:
# **** Headline is renamed to title
# **** AnswerLanguage has been removed 
# **** Encoding has been removed, since it can be retrieved from the Language
# **** Category is implicit from parent

manage_addQuestionForm=Globals.DTMLFile('dtml/questionAdd', globals())
def manage_addQuestion(self, id='', title='', Question='', Language='', InquirersName='', InquirersEmail='',
        ExpertId='', isPrivate=0, Answer='', Rating='', Timelineness='', askedDateTime=None, answeredDateTime=None, 
        ReturnURL='', DestinationURL='', REQUEST=None):
    """ Add a Question to a folder
        The purpose is allow the Administrator to add questions
        Therefore the question and answer is filled out at the same time
    """
    l_errors = []
    #validate form data
    if not title:
        l_errors.append(Resources.ERR_MSG_TITLE_REQUIRED)
    if not InquirersName:
        l_errors.append(Resources.ERR_MSG_NAME_REQUIRED)
    if not InquirersEmail:
        l_errors.append(Resources.ERR_MSG_VALIDEMAIL_REQUIRED)
    if not ExpertId:
        l_errors.append(Resources.ERR_MSG_EXPERTID_REQUIRED)
    if not Language:
        l_errors.append(Resources.ERR_MSG_LANGUAGE_REQUIRED)
    if not Question:
        l_errors.append(Resources.ERR_MSG_QUESTION_REQUIRED)
    if isPrivate:
        isPrivate = 1
    else:
        isPrivate = 0
    #check for errors
    if len(l_errors)>0:
        if REQUEST:
            self.setSessionQuestion(id, title, InquirersName, InquirersEmail, ExpertId, Language, Question, isPrivate, Rating, Answer, askedDateTime)
            self.setSessionErrors(l_errors)
            if ReturnURL:
                return REQUEST.RESPONSE.redirect(ReturnURL)
            else:
                return REQUEST.RESPONSE.redirect('manage_addQuestionForm')
        else:
            return None
    else:
        if not id:
            id = Resources.EXPERT_QUESTION_DEFAULT_PREFIX + str(int(DateTime.DateTime()))
        askedDateTime = DateTime.DateTime()
        access_code = self.utGenRandomKey()
        quest = ExpertQuestion(id, title, Question, Language, InquirersName, InquirersEmail, ExpertId, isPrivate, Answer, Rating, Timelineness, askedDateTime, answeredDateTime, access_code)
        self._setObject(id, quest)
        self.delSessionQuestion()
        self.delSessionErrors()
        #send email notification
        l_expert_obj = self.getEnroledExpert(ExpertId)
        self.newQuestionEmail(l_expert_obj.Email, self.adminEmail, l_expert_obj.Firstname + ' ' + l_expert_obj.Lastname, self.getEngine().absolute_url(0) + '/login_html?name=' + ExpertId)
        if REQUEST:
            if DestinationURL:
                return REQUEST.RESPONSE.redirect(DestinationURL)
            else:
                return self.manage_main(self,REQUEST)


class ExpertQuestion(CatalogAware, SimpleItem, Utils):
    """ Questions for Experts in BesserWisser """

    meta_type = Resources.METATYPE_EXPERT_QUESTION
    icon = 'misc_/BesserWisser/question.gif'

    security = ClassSecurityInfo()

    manage_options=(
        (
            {'label':'Properties', 'action':'manage_main', 'help' :('BesserWisser','Question_Properties.stx')},
            {'label':'View', 'action':'index_html' },
        )
        +
        SimpleItem.manage_options
        +
        AccessControl.Role.RoleManager.manage_options
    )

    def __init__(self, id, title, Question, Language, InquirersName, InquirersEmail, 
            ExpertId, isPrivate, Answer, Rating, Timelineness, askedDateTime, answeredDateTime, access_code):
        """ Initialize the question object """
        self.id = id
        self.title = title
        self.Question = Question
        self.Language = Language
        self.InquirersName = InquirersName
        self.InquirersEmail = InquirersEmail
        self.ExpertId = ExpertId
        self.isPrivate = isPrivate
        self.Answer = Answer
        self.Rating = Rating
        self.Timelineness = Timelineness
        self.askedDateTime = askedDateTime
        self.answeredDateTime = answeredDateTime
        self.access_code = access_code

    def __setstate__(self,state):
        """ """
        ExpertQuestion.inheritedAttribute('__setstate__')(self, state)

    security.declareProtected('Change Questions', 'manage_editQuestion')
    def manage_editQuestion(self, title='', Question='', Language='', InquirersName='', InquirersEmail='', 
            ExpertId='', isPrivate=0, Answer='', Rating='', Timelineness='', askedDateTime='', REQUEST=None):
        """ Change the question by the Administrator
            All fields should be changeable """
        l_errors = []
        #validate form data
        if not title:
            l_errors.append(Resources.ERR_MSG_TITLE_REQUIRED)
        if not InquirersName:
            l_errors.append(Resources.ERR_MSG_NAME_REQUIRED)
        if not InquirersEmail:
            l_errors.append(Resources.ERR_MSG_VALIDEMAIL_REQUIRED)
        if not ExpertId:
            l_errors.append(Resources.ERR_MSG_EXPERTID_REQUIRED)
        if not Language:
            l_errors.append(Resources.ERR_MSG_LANGUAGE_REQUIRED)
        if not Question:
            l_errors.append(Resources.ERR_MSG_QUESTION_REQUIRED)
        if isPrivate:
            isPrivate = 1
        else:
            isPrivate = 0
        #check for errors
        if len(l_errors)>0:
            if REQUEST:
                self.setSessionQuestion(self.id, title, InquirersName, InquirersEmail, ExpertId, Language, Question, isPrivate, Rating, Answer, askedDateTime)
                self.setSessionErrors(l_errors)
                return REQUEST.RESPONSE.redirect('manage_main')
            else:
                return None
        else:
            self.title = title
            self.Question = Question
            self.Language = Language
            self.InquirersName = InquirersName
            self.InquirersEmail = InquirersEmail
            self.ExpertId = ExpertId
            self.isPrivate = int(isPrivate)
            self.Answer = Answer
            self.Rating = Rating
            self.Timelineness = Timelineness
            self.askedDateTime = askedDateTime
            self._p_changed = 1
            self.delSessionQuestion()
            self.delSessionErrors()
            self.recatalogBesserWisserObject(self)
            if REQUEST:
                message="Question Changed"
                return self.manage_main(self,REQUEST,manage_tabs_message=message)

    security.declarePrivate('manage_afterAdd')
    def manage_afterAdd(self, item, container):
        """ This method is called, whenever _setObject in ObjectManager gets
        called. This is the case after a normal add and if the object is a
        result of cut-paste- or rename-operation """
        self.catalogBesserWisserObject(self)

    security.declarePrivate('manage_beforeDelete')
    def manage_beforeDelete(self, item, container):
        """ This method is called, when the object is deleted """
        self.uncatalogBesserWisserObject(self)

    security.declarePublic('getEncodingFromLanguage')
    def getEncodingFromLanguage(self):
        """ Returns the encoding for a language """
        return self.getLanguageEncoding(self.Language)

    security.declarePublic('isQuestionAnswered')
    def isQuestionAnswered(self):
        """ Returns true if the question is answered, false otherwise """
        return self.answeredDateTime is not None

    security.declarePublic('isQuestionRated')
    def isQuestionRated(self):
        """ Returns true if the question is rated, false otherwise """
        return self.Rating != ''

    security.declarePublic('canRateQuestion')
    def canRateQuestion(self, p_access_code):
        """ Returns true if the question can be rated: the access_code provided as parameter
            is equal with question's access_code property """
        return self.access_code == p_access_code

    security.declarePrivate('getTimelineness')
    def getTimelineness(self):
        """ Calculates the timeliness for a question """
        if self.answeredDateTime is not None:
            l_t = self.answeredDateTime - self.askedDateTime
            if l_t < 0:
                return 0
            elif l_t < 1:
                return 5
            elif l_t > 1 and l_t < 1.5:
                return 4
            elif l_t > 1.5 and l_t < 2 :
                return 3
            elif l_t > 2 and l_t < 3:
                return 2
            elif l_t > 3:
                return 1
        else: 
            return 0

    def answerQuestion(self, Answer='', REQUEST=None):
        """ Answer question """
        # answer question
        self.Answer = str(Answer).strip()
        self.answeredDateTime = DateTime.DateTime()
        self.Timelineness = self.getTimelineness()
        self._p_changed = 1
        # update BesserWisser catalog
        self.recatalogBesserWisserObject(self)
        # send email notification to question inquirer
        self.answeredQuestionEmail(self.InquirersEmail, self.adminEmail, self.InquirersName, self.title, self.absolute_url(0) + '/rate_html?accesscode=' + self.access_code)
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.getEnroledExpert(self.REQUEST.AUTHENTICATED_USER.getUserName()).absolute_url(0) + '/answerquestions_html')

    def rateQuestion(self, rating_value='3', accesscode_value=None, REQUEST=None):
        """ Rate question """
        if not self.isQuestionRated():
            if self.canRateQuestion(accesscode_value):
                # ok to rate this question
                try:
                    rating_value = int(rating_value)
                except:
                    rating_value = 3
                self.Rating = rating_value
                self._p_changed = 1
        if REQUEST:
            REQUEST.RESPONSE.redirect(self.absolute_url(0) + '/index_html')

    security.declarePrivate('displayRatingTimelineness')
    def displayRatingTimelineness(self):
        """ Returns a string Rating/Timelineness """
        res = '-/-'
        if self.answeredDateTime is not None:
            if self.answeredDateTime.greaterThan(DateTime.DateTime('01/01/1990')):
                if self.Rating != '':
                    l_Rating = self.Rating
                else:
                    l_Rating = '-'
                if self.Timelineness != '':
                    l_Timelineness = self.Timelineness
                else:
                    l_Timelineness = '-'
                res = str(l_Rating) + '/' + str(l_Timelineness)
            else:
                res = 'No rating'
        return res

    security.declareProtected(view_management_screens, 'manage_main')
    manage_main = Globals.DTMLFile('dtml/Question_edit', globals())

    security.declareProtected(view, 'index_html')
    index_html = Globals.DTMLFile('dtml/Question_view', globals())

    security.declareProtected(view, 'answer_html')
    answer_html = Globals.DTMLFile('dtml/Question_answer', globals())

    security.declareProtected(view, 'rate_html')
    rate_html = Globals.DTMLFile('dtml/Question_rate', globals())


Globals.InitializeClass(ExpertQuestion)
