# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania

# Python imports

# Zope imports

# Product imports

#constants
_SESSION_ERRORS = "site_errors"
_SESSION_INFO = "site_infos"
_SESSION_EXPERTFORMDATA = "site_expertformdata"
_SESSION_ADDQUESTION_ID = "site_question_id"
_SESSION_ADDQUESTION_TITLE = "site_question_title"
_SESSION_ADDQUESTION_INQUIRERSNAME = "site_question_InquirersName"
_SESSION_ADDQUESTION_INQUIRERSEMAIL = "site_question_InquirersEmail"
_SESSION_ADDQUESTION_EXPERTID = "site_question_ExpertId"
_SESSION_ADDQUESTION_LANGUAGE = "site_question_Language"
_SESSION_ADDQUESTION_QUESTION = "site_question_Question"
_SESSION_ADDQUESTION_PRIVATE = "site_question_isPrivate"
_SESSION_ADDQUESTION_RATING = "site_question_Rating"
_SESSION_ADDQUESTION_ANSWER = "site_question_Answer"
_SESSION_ADDQUESTION_ASKEDDATETIME = "site_question_askedDateTime"

class SessionManager:
    """This class has some methods to work with session variables"""

    def __init__(self):
        """Constructor"""
        pass

    def __isSession(self, key):
        """Test if exists a variable with the given key in SESSION"""
        return self.REQUEST.SESSION.has_key(key)

    def __getSession(self, key, default):
        """Get a key value from SESSION; if that key doesn't exist then return default value"""
        try:
            return self.REQUEST.SESSION[key]
        except:
            return default

    def __setSession(self, key, value):
        """Add a value to SESSION"""
        try:
            self.REQUEST.SESSION.set(key, value)
        except:
            pass

    def __delSession(self, key):
        """Delete a value from SESSION"""
        try:
            self.REQUEST.SESSION.delete(key)
        except:
            pass


    #Public methods
    def getSessionValue(self, key, default):
        """ Returns the session value for one key """
        return self.__getSession(key, default)

    def setSession(self, key, value):
        """ Set the session value for key """
        return self.__setSession(key, value)

    def delSession(self, key):
        """ Delete a key from session """
        return self.__delSession(key)

    def isSession(self, key):
        """ Returns true if this key exists in session """
        return self.__isSession(key)

    #manage information
    def isSessionInfo(self):
        """Returns true if this key exists"""
        return self.__isSession(_SESSION_INFO)

    def getSessionInfo(self, default=None):
        """Returns the session value for errors"""
        return self.__getSession(_SESSION_INFO, default)

    def setSessionInfo(self, value):
        """Set the session value for errors"""
        self.__setSession(_SESSION_INFO, value)

    def delSessionInfo(self):
        """Delete a key"""
        self.__delSession(_SESSION_INFO)

    #manage errors
    def isSessionErrors(self):
        """Returns true if this key exists"""
        return self.__isSession(_SESSION_ERRORS)

    def getSessionErrors(self, default=None):
        """Returns the session value for errors"""
        return self.__getSession(_SESSION_ERRORS, default)

    def setSessionErrors(self, value):
        """Set the session value for errors"""
        self.__setSession(_SESSION_ERRORS, value)

    def delSessionErrors(self):
        """Delete a key"""
        self.__delSession(_SESSION_ERRORS)

    #expertdata object
    def isSessionExpertFormData(self):
        """ """
        return self.__isSession(_SESSION_EXPERTFORMDATA)

    def getSessionExpertFormData(self, default=None):
        """ """
        return self.__getSession(_SESSION_EXPERTFORMDATA, default)

    def setSessionExpertFormData(self, value):
        """ """
        self.__setSession(_SESSION_EXPERTFORMDATA, value)

    def delSessionExpertFormData(self):
        """ """
        self.__delSession(_SESSION_EXPERTFORMDATA)

    #question
    def setSessionQuestion(self, p_id, p_title, p_InquirersName, p_InquirersEmail, p_ExpertId, p_Language, p_Question, p_isPrivate, p_Rating, p_Answer, p_askedDateTime):
        """ """
        self.__setSession(_SESSION_ADDQUESTION_ID, p_id)
        self.__setSession(_SESSION_ADDQUESTION_TITLE, p_title)
        self.__setSession(_SESSION_ADDQUESTION_INQUIRERSNAME, p_InquirersName)
        self.__setSession(_SESSION_ADDQUESTION_INQUIRERSEMAIL, p_InquirersEmail)
        self.__setSession(_SESSION_ADDQUESTION_EXPERTID, p_ExpertId)
        self.__setSession(_SESSION_ADDQUESTION_LANGUAGE, p_Language)
        self.__setSession(_SESSION_ADDQUESTION_QUESTION, p_Question)
        self.__setSession(_SESSION_ADDQUESTION_PRIVATE, p_isPrivate)
        self.__setSession(_SESSION_ADDQUESTION_RATING, p_Rating)
        self.__setSession(_SESSION_ADDQUESTION_ANSWER, p_Answer)
        self.__setSession(_SESSION_ADDQUESTION_ASKEDDATETIME, p_askedDateTime)

    def delSessionQuestion(self):
        """ """
        self.__delSession(_SESSION_ADDQUESTION_ID)
        self.__delSession(_SESSION_ADDQUESTION_TITLE)
        self.__delSession(_SESSION_ADDQUESTION_INQUIRERSNAME)
        self.__delSession(_SESSION_ADDQUESTION_INQUIRERSEMAIL)
        self.__delSession(_SESSION_ADDQUESTION_EXPERTID)
        self.__delSession(_SESSION_ADDQUESTION_LANGUAGE)
        self.__delSession(_SESSION_ADDQUESTION_QUESTION)
        self.__delSession(_SESSION_ADDQUESTION_PRIVATE)
        self.__delSession(_SESSION_ADDQUESTION_RATING)
        self.__delSession(_SESSION_ADDQUESTION_ANSWER)
        self.__delSession(_SESSION_ADDQUESTION_ASKEDDATETIME)

    def getSessionQuestionId(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_ID, p_default)

    def getSessionQuestionTitle(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_TITLE, p_default)

    def getSessionQuestionInquirersName(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_INQUIRERSNAME, p_default)

    def getSessionQuestionInquirersEmail(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_INQUIRERSEMAIL, p_default)

    def getSessionQuestionExpertId(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_EXPERTID, p_default)

    def getSessionQuestionLanguage(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_LANGUAGE, p_default)

    def getSessionQuestionQuestion(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_QUESTION, p_default)

    def getSessionQuestionPrivate(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_PRIVATE, p_default)

    def getSessionQuestionRating(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_RATING, p_default)

    def getSessionQuestionAnswer(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_ANSWER, p_default)

    def getSessionQuestionAskedDateTime(self, p_default=''):
        """ """
        return self.__getSession(_SESSION_ADDQUESTION_ASKEDDATETIME, p_default)
