# The contents of this file are subject to the Mozilla Public
# License Version 1.1 (the "License"); you may not use this file
# except in compliance with the License. You may obtain a copy of
# the License at http://www.mozilla.org/MPL/
#
# Software distributed under the License is distributed on an "AS
# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
# implied. See the License for the specific language governing
# rights and limitations under the License.
#
# The Original Code is BesserWisser version 2.0.
#
# The Initial Developer of the Original Code is European Environment
# Agency (EEA).  Portions created by Finsiel Romania are
# Copyright (C) European Environment Agency.  All
# Rights Reserved.
#
# Contributor(s):
# Soren Roug, EEA
# Miruna Badescu, Finsiel Romania

""" Useful functions """

# Python imports
import string
from whrandom import choice
import operator

# Zope imports

# Product imports

# aliases
_alias_operator_getitem = operator.getitem

class Utils:
    """ Usefull functions for this product """

    def __init__(self):
        """ """
        pass

    def utsetFormError(self, req, key, msg):
        """ Used for validation errors """
        req.set('FORM_ERROR', 1)
        req.set('FORM_ERROR_' + key, msg)
        return req

    def utGenRandomId(self, p_length=10, p_chars=string.digits):
        """Generate a random numeric id."""
        return ''.join([choice(p_chars) for i in range(p_length)])

    def utGenRandomKey(self, p_length=10, p_chars=string.letters + string.digits):
        """Generate a random alpha-numeric key."""
        return ''.join([choice(p_chars) for i in range(p_length)])

    def utBuildEmailList(self, p_emails):
        """Build a string with all emails"""
        if len(p_emails)>0:
            return '<' + '>,\n<'.join(p_emails) + '>'
        else:
            return ''

    #################
    #   LISTS STUFF #
    #################
    def utConvertToList(self, something):
        """Convert to list"""
        ret = something
        if type(something) is type(''):
            ret = [something]
        return ret

    def utSplitToList(self, something, separator='/'):
        """Get a string like value1<separator>value2..., and returns a list [value1, values...]"""
        if something == '':
            return []
        else:
            return something.split(separator)

    def utConvertLinesToList(self, value):
        """Takes a value from a textarea control and returns a list of values"""
        if type(value) == type([]):
            return value
        if value == '':
            return []
        else:
            values = []
            for v in value.split('\r\n'):
                if v != '':
                    values.append(v)
        return values

    def utConvertListToLines(self, values):
        """Takes a list of values and returns a value for a textarea control"""
        if len(values) == 0:
            return ''
        else:
            return '\r\n'.join(values)

    def utLookupForUser(self, userid):
        """ lookup for an user id in all acl_users from hierarchy"""
        for obj in self.superValues('User Folder'):
            if obj.getUserById(userid):
                return 0
        return 1

    def utListDifference(self, p_l1, p_l2):
        """return a list with elements from p_l1 that are not in p_l2"""
        return [l_e1 for l_e1 in p_l1 if l_e1 not in p_l2]

    def utListIntersection(self, p_l1, p_l2):
        """return the intersection of two lists"""
        return [l_e1 for l_e1 in p_l1 if l_e1 in p_l2]

    def utSortObjsListByAttr(self, p_list, p_attr, p_desc=1):
        """ Sort a list of objects by an attribute values """
        l_len = len(p_list)
        l_temp = map(None, map(getattr, p_list, (p_attr,)*l_len), xrange(l_len), p_list)
        l_temp.sort()
        if p_desc:
            l_temp.reverse()
        return map(_alias_operator_getitem, l_temp, (-1,)*l_len)

    def utFilterObjsListByAttr(self, p_list, p_attr, p_value):
        """ Filter a list of objects by an attribute value """
        l_len = len(p_list)
        l_temp = map(None, map(getattr, p_list, (p_attr,)*l_len), (p_value,)*l_len, p_list)
        l_temp = filter(lambda x: x[0]==x[1], l_temp)
        return map(_alias_operator_getitem, l_temp, (-1,)*len(l_temp))

    #####################
    #   STRING STUFF    #
    #####################
    def utStrStrip(self, p_value):
        """removes leadeaing and trailing white spaces"""
        return string.strip(p_value)

    def utStrSplit(self, p_value, p_separator):
        """splits a string"""
        return string.split(p_value, p_separator)

    def utStrJoin(self, p_list, p_separator):
        """joins a list of strings"""
        return string.join(p_list, p_separator)

    def utStrReplace(self, p_string, p_old, p_new):
        """replace"""
        return string.replace(p_string, p_old, p_new)

    #################
    #   FORMS STUFF #
    #################
    def utAddObjectAction(self, REQUEST=None):
        """Check if adding an object"""
        try:
            return REQUEST.has_key('add')
        except:
            return 0

    def utUpdateObjectAction(self, REQUEST=None):
        """Check if updating an object"""
        try:
            return REQUEST.has_key('update')
        except:
            return 0

    def utDeleteObjectAction(self, REQUEST=None):
        """Check if deleting an object"""
        try:
            return REQUEST.has_key('delete')
        except:
            return 0

    def utImportObjectAction(self, REQUEST=None):
        """Check if import action"""
        try:
            return REQUEST.has_key('import')
        except:
            return 0

    def utRefreshObjectAction(self, REQUEST=None):
        """Check if refresh action"""
        try:
            return REQUEST.has_key('refresh')
        except:
            return 0

    #################
    #   ENCODING    #
    #################
    def utJsEncode(self, p_string):
        """Encode a string for javascript processing"""
        l_tmp = p_string
        l_tmp = l_tmp.replace('\\', '\\\\')
        l_tmp = l_tmp.replace('\'', '\\\'')
        l_tmp = l_tmp.replace('\"', '\\\"')
        return l_tmp

    def utAsciiEncode(self, p_value):
        """Gets a value and returns a string"""
        if p_value is not None:
            return unicode(str(p_value), 'latin-1').encode('ascii', 'replace')
        else:
            return ''
